@@ -1,63 +1,44 @@
 """SatNOGS Network API django rest framework Views"""
+from django.conf import settings
 from django.core.exceptions import ObjectDoesNotExist
+from django.db import transaction
 from django.db.models import Count
+from django.db.models.query import QuerySet
 from django.shortcuts import get_object_or_404
 from django.utils.timezone import now
 from rest_framework import mixins, status, viewsets
 from rest_framework.response import Response
-from rest_framework.schemas.openapi import AutoSchema
-from rest_framework.schemas.utils import is_list_view
 from rest_framework.serializers import ValidationError
 
 from network.api import filters, pagination, serializers
 from network.api.perms import StationOwnerPermission
 from network.base.models import Observation, Station
 from network.base.rating_tasks import rate_observation
-from network.base.tasks import sync_to_db
+from network.base.tasks import delay_task_with_lock, process_audio, sync_to_db
 from network.base.validators import NegativeElevationError, NoTleSetError, \
     ObservationOverlapError, SinglePassError
 
 
-class ViewSchema(AutoSchema):
-    """
-    AutoSchema override
-    """
-    def _get_operation_id(self, path, method):
-        """
-        Compute an operation ID from the view name.
-        """
-        method_name = getattr(self.view, 'action', method.lower())
-        if is_list_view(path, method, self.view):
-            action = 'list'
-        elif method_name not in self.method_mapping:
-            action = method_name
-        else:
-            action = self.method_mapping[method.lower()]
-
-        name = self.view.__class__.__name__
-        if name.endswith('APIView'):
-            name = name[:-7]
-        elif name.endswith('View'):
-            name = name[:-4]
-
-        if name.endswith(action.title()):
-            name = name[:-len(action)]
-
-        if action == 'list' and not name.endswith('s'):
-            name += 's'
-
-        return action + name
-
-
 class ObservationView(  # pylint: disable=R0901
         mixins.ListModelMixin, mixins.RetrieveModelMixin, mixins.UpdateModelMixin,
         mixins.CreateModelMixin, viewsets.GenericViewSet):
     """SatNOGS Network Observation API view class"""
-    queryset = Observation.objects.prefetch_related('satellite', 'demoddata', 'ground_station')
     filterset_class = filters.ObservationViewFilter
     permission_classes = [StationOwnerPermission]
     pagination_class = pagination.LinkedHeaderPageNumberPagination
 
+    def get_queryset(self):
+        if self.action == 'update':
+            queryset = Observation.objects.select_for_update()
+        else:
+            queryset = Observation.objects.prefetch_related(
+                'satellite', 'demoddata', 'ground_station'
+            )
+        if isinstance(queryset, QuerySet):
+            # Ensure queryset is re-evaluated on each request.
+            queryset = queryset.all()
+        return queryset
+
     def get_serializer_class(self):
         """Returns the right serializer depending on http method that is used"""
         if self.action == 'create':
@@ -86,39 +67,50 @@ class ObservationView(  # pylint: disable=R0901
 
     def update(self, request, *args, **kwargs):
         """Updates observation with audio, waterfall or demoded data"""
-        instance = self.get_object()
-        if request.data.get('client_version'):
-            instance.ground_station.client_version = request.data.get('client_version')
-            instance.ground_station.save()
-        if request.data.get('demoddata'):
-            try:
-                file_path = 'data_obs/{0}/{1}'.format(instance.id, request.data.get('demoddata'))
-                instance.demoddata.get(payload_demod=file_path)
-                return Response(
-                    data='This data file has already been uploaded',
-                    status=status.HTTP_403_FORBIDDEN
-                )
-            except ObjectDoesNotExist:
-                demoddata = instance.demoddata.create(payload_demod=request.data.get('demoddata'))
-                sync_to_db.delay(frame_id=demoddata.id)
-        if request.data.get('waterfall'):
-            if instance.has_waterfall:
-                return Response(
-                    data='Watefall has already been uploaded', status=status.HTTP_403_FORBIDDEN
-                )
-        if request.data.get('payload'):
-            if instance.has_audio:
-                return Response(
-                    data='Audio has already been uploaded', status=status.HTTP_403_FORBIDDEN
-                )
+        with transaction.atomic():
+            instance = self.get_object()
+            if request.data.get('client_version'):
+                instance.ground_station.client_version = request.data.get('client_version')
+                instance.ground_station.save()
+            if request.data.get('demoddata'):
+                try:
+                    file_path = 'data_obs/{0}/{1}'.format(
+                        instance.id, request.data.get('demoddata')
+                    )
+                    instance.demoddata.get(payload_demod=file_path)
+                    return Response(
+                        data='This data file has already been uploaded',
+                        status=status.HTTP_403_FORBIDDEN
+                    )
+                except ObjectDoesNotExist:
+                    demoddata = instance.demoddata.create(
+                        payload_demod=request.data.get('demoddata')
+                    )
+                    sync_to_db.delay(frame_id=demoddata.id)
+            if request.data.get('waterfall'):
+                if instance.has_waterfall:
+                    return Response(
+                        data='Watefall has already been uploaded',
+                        status=status.HTTP_403_FORBIDDEN
+                    )
+            if request.data.get('payload'):
+                if instance.has_audio:
+                    return Response(
+                        data='Audio has already been uploaded', status=status.HTTP_403_FORBIDDEN
+                    )
+
+            # False-positive no-member (E1101) pylint error:
+            # Parent class rest_framework.mixins.UpdateModelMixin provides the 'update' method
+            super().update(request, *args, **kwargs)  # pylint: disable=E1101
 
-        # False-positive no-member (E1101) pylint error:
-        # Parent class rest_framework.mixins.UpdateModelMixin provides the 'update' method
-        super(ObservationView, self).update(request, *args, **kwargs)  # pylint: disable=E1101
         if request.data.get('waterfall'):
             rate_observation.delay(instance.id, 'waterfall_upload')
         if request.data.get('demoddata'):
             rate_observation.delay(instance.id, 'data_upload')
+        if request.data.get('payload'):
+            delay_task_with_lock(
+                process_audio, instance.id, settings.PROCESS_AUDIO_LOCK_EXPIRATION, instance.id
+            )
         return Response(status=status.HTTP_200_OK)
 
 
@@ -141,7 +133,6 @@ class TransmitterView(  # pylint: disable=R0901
     lookup_field = 'transmitter_uuid'
     filterset_class = filters.TransmitterViewFilter
     pagination_class = pagination.LinkedHeaderPageNumberPagination
-    schema = ViewSchema()
 
 
 class JobView(viewsets.ReadOnlyModelViewSet):  # pylint: disable=R0901
@@ -150,7 +141,6 @@ class JobView(viewsets.ReadOnlyModelViewSet):  # pylint: disable=R0901
     serializer_class = serializers.JobSerializer
     filterset_class = filters.ObservationViewFilter
     filterset_fields = ('ground_station')
-    schema = ViewSchema()
 
     def get_queryset(self):
         """Returns queryset for Job API view"""
