@@ -126,7 +126,7 @@ def export_station_status(self, request, queryset):
 
 
 def community_get_discussion_details(
-        observation_id, satellite_name, norad_cat_id, observation_url
+    observation_id, satellite_name, norad_cat_id, observation_url
 ):
     """
     Return the details of a discussion of the observation (if existent) in the
@@ -181,7 +181,9 @@ def sync_demoddata_to_db(frame):
         'longitude': ground_station.lng,
         'latitude': ground_station.lat,
         'frame': frame.display_payload_hex().replace(' ', ''),
-        'satnogs_network': 'True'  # NOT a part of SiDS
+        'satnogs_network': 'True',  # NOT a part of SiDS
+        'observation_id': obs.id,  # NOT a part of SiDS
+        'station_id': obs.ground_station.id  # NOT a part of SiDS
     }
 
     telemetry_url = "{}telemetry/".format(settings.DB_API_ENDPOINT)
